#!/usr/bin/env ruby

require "bundler/inline"

gemfile do
  source "https://rubygems.org"
  gem "capybara"
  gem "cuprite", "~> 0.15" # not sure why I need this pin
  gem "debug"
  gem "ferrum"
  gem "logger"
end

require "logger"
require "net/http"

require "capybara/cuprite"
Capybara.javascript_driver = :cuprite
Capybara.register_driver(:cuprite) do |app|
  Capybara::Cuprite::Driver.new(app, headless: false)
end
Capybara.current_driver = :cuprite

require "capybara/dsl"
# Create a session instead of including the DSL module
session = Capybara::Session.new(:cuprite)

Book = Data.define(:id, :title)

AMAZON_EMAIL = ENV.fetch("AMAZON_EMAIL")
AMAZON_PASSWORD = ENV.fetch("AMAZON_PASSWORD")

def downloaded_books
  Dir[File.expand_path("~/Downloads/*")]
    .select { |file| file =~ /\.(azw3?|tpz)$/ } # tpz?!
end

def wait_for_download
  already_downloaded = downloaded_books
  yield
  loop do
    current = downloaded_books
    new_book, *rest = current - already_downloaded
    if new_book.nil?
      sleep 0.1
    else
      fail unless rest.empty?
      return new_book
    end
  end
end

log = Logger.new(STDOUT)

url = "https://www.amazon.com/hz/mycd/digital-console/contentlist/booksPurchases/dateDsc"
session.visit url

session.fill_in "email", with: AMAZON_EMAIL
session.fill_in "password", with: AMAZON_PASSWORD
session.find("#signInSubmit").click
binding.irb

# I could use pstore here or something, but this is
# pretty damn convenient
files = downloaded_books
ids = files.map { |file|
  `strings "#{file}" | grep -E "\\bB[A-Z0-9]{9}\\b" | head -n1`.chomp
}
log.info("Already downloaded: #{files.zip(ids)}")

# this could be smarter, but I am lazy
(1..33).each do |page_number|
  url = "https://www.amazon.com/hz/mycd/digital-console/contentlist/booksPurchases/dateDsc?pageNumber=#{page_number}"
  session.visit url

  books = session.all(".digital_entity_title").map { |element|
    Book.new(id: element["id"].split(?-).last, title: element.text)
  }.map.with_index.reject {|book, _| ids.include?(book.id) }

  log.info("Downloading: #{books.map(&:first).map(&:title)}")

  books.each do |book, i|
    begin
      # kindle and download don't seem to ever be
      # visible, so do this awkward dance to actually
      # associate them to the right book
      
      more_actions, kindle, download = session.all(:xpath, "//span[text()='More actions']").zip(
        session.all('span', id: /download_and_transfer_list_.*_0/, visible: false),
        session.all("div[id$='_CONFIRM']", text: 'Download', visible: false),
      ).fetch(i)

      more_actions.base.node.scroll_into_view
      more_actions.click


      session.find(:xpath, "//span[text()='Download & transfer via USB']").click

      # click on first kindle
      kindle.click

      log.info("Downloading #{book.title} (#{book.id})")

      # trouble with getting the download button to click

      # I'd use session.driver.browser.page.downloads.wait
      # here, but it (and also .files) don't seem to
      # work quite right
      downloaded = wait_for_download { download.click }

      session.find("#notification-close").click
    rescue StandardError => e
      log.error("Error processing book #{book.title} (#{book.id}): #{e.message}")
      log.error(e.backtrace.join("\n"))
    end
  end
end
